#include "LampShow.h"
#include "MultiballLampShowWorker.h"
#include "FileLampShowWorker.h"

LampShow::LampShow(uint8_t lampState[], uint8_t outputPort[]) {
  _pLampState = lampState;
  _pOutputPort = outputPort;
  for (uint8_t i = 0; i < MAX_WORKERS; i++) {
    _pWorkerArray[i] = nullptr;
  }
}

LampShow::~LampShow() {
  //dtor
}

void LampShow::finishedLampShow(LampShowWorker *lampShowWorker) {
  for (uint8_t i = 0; i < MAX_WORKERS; i++) {
    if (_pWorkerArray[i] != nullptr && _pWorkerArray[i] == lampShowWorker) {
      delete _pWorkerArray[i];
      _pWorkerArray[i] = nullptr;
    }
  }
}

void LampShow::schedule() {
  Scheduler::schedule();
  for (uint8_t i = 0; i < MAX_WORKERS; i++) {
    if (_pWorkerArray[i] != nullptr) {
      _pWorkerArray[i]->schedule();
    }
  }
}
void LampShow::playLampShow(Sequence sequence) {
  // If all workers are full, we cannot
  // start a new one else start one.
  int i = -1;
  for (uint8_t index = 0; index < MAX_WORKERS; index++) {
    if (_pWorkerArray[index] == nullptr) {
      // We found an empty slot!
      i = index;
      break;
    }
  }

  if (i < 0) {
    return;
  }

  switch (sequence) {
    case LampShow::Sequence::Multiball:
      _pWorkerArray[i] = new MultiballLampShowWorker(this, true);
      break;

    case LampShow::Sequence::BallPlunged:
      _pWorkerArray[i] = new FileLampShowWorker(this, true);
      break;

    case LampShow::Sequence::Jackpot:
      break;

    case LampShow::Sequence::SuperJackpot:
      break;
  }
}

uint8_t *LampShow::getLampStateArray() {
  return _pLampState;
}

uint8_t *LampShow::getOutputPortArray() {
  return _pOutputPort;
}
